SUBROUTINE albedo_calc(balance,cum_PDDs,d_index,b_albedo, &
			seasonablation,snowday,modelday,solel,diffuse_frac,albedo)
USE nrtype
USE params
IMPLICIT NONE

!	This is just a subroutinized version of the albedo model already
!	written into the melturice code. Takes in the DEM of the glacier
!	the mass balance array, PDD count, and the albedo array and adjusts the 
!	albedo based on the new PDDs.

	REAL(DP),		DIMENSION(1:nlayers),		INTENT(IN   )		:: balance, &
																   cum_pdds, &
																   seasonablation, &
																   snowday
	INTEGER(I4B),							INTENT(IN   )		:: d_index
	REAL(DP),								INTENT(IN   )		:: b_albedo
	REAL(DP),								INTENT(IN   )		:: modelday
	REAL(DP),		DIMENSION(1:24),		INTENT(IN   )		:: solel
	REAL(DP),								INTENT(IN   )		:: diffuse_frac
	REAL(DP),								INTENT(INOUT)		:: albedo
	
	REAL(DP),		DIMENSION(2)								:: a
	REAL(DP)													:: top, &
																   bottom, &
																   alb, &
																   snowdepth, &
																   cloud_frac, &
																   r_vis, &
																   r_nir, &
																   melt_frac=0._dp


	INTEGER(I4B)												:: i


	SELECT CASE (albedo_model)
	
	CASE(1)   !Colin's Original
		IF (balance(1) .GT. 0._dp) THEN 
			snowdepth=SUM(balance(1:nlayers))
		ELSE
			snowdepth=SUM(balance(2:nlayers))
		ENDIF
		alb=0.53+(a_fresh-0.53)*exp((snowday(d_index)-modelday)/a6)
		alb=alb+(b_albedo-alb)*exp(-1*snowdepth/a5)-(a7*seasonablation(d_index))
		IF ((snowdepth .GT. 0.2) .AND. (alb .LT. min_snowalbedo)) alb=min_snowalbedo
		IF (alb.lt.(b_albedo))   alb=b_albedo
		IF (alb.lt.0.12) alb=0.12
		albedo=alb
	CASE (2)	!This will be the wiscombe, marks, jordan, arendt model
!**********************************************************************************************				
!this model is the more sophisticated albedo model that I've been promising
!To code up for a while now. It's going to calculate grain size dependence 
!of dual band albedo -- NIR and VIS. Grain growth will be a function of time
!only, but effective grain growth due to saturation of snow will also be
!included. It's a relatively simple model, but liquid water content and 
!contamination may be doubly tricky issues to resolve. They may require 
!enough parameterization that the physicality of this model are lost to 
!guesswork. For debris content, maybe calculate an intial amount of crap
!in the snow based on elevation and total snow pack. As this ablates, the
!junk accumulates at the top where it will be important for affecting 
!albedo. This is visually constrained on alpine glaciers. As for water content,
!arendt gives an intial guess offering the possibility to double effective
!grain size. I'm guessing that this will be the permanent state of an
!alpine glacier snowpack, where refreezing at night has little effect on 
!albedo measurements. So, a one way albedo decrease as melt ensues. 
!**********************************************************************************************
		cloud_frac=(diffuse_frac-0.2_dp)/0.8_dp
		IF (d_index .GE. 2) THEN !Snow on snow
			IF (balance(d_index) .GT. 0._dp) THEN !Snow on snow
				DO i=1,2,1
					melt_frac=MIN(1._dp,seasonablation((d_index-(i-1)))/balance((d_index-(i-1))))
					CALL grainsize((modelday-snowday((d_index-(i-1)))),melt_frac,r_vis,r_nir)
					IF (tmstp .EQ. 86400) THEN
						CALL albedo_compute(solel(12),diffuse_frac,cloud_frac,r_vis,r_nir,a(i))
					ELSE
						CALL albedo_compute(solel(1),diffuse_frac,cloud_frac,r_vis,r_nir,a(i))
					ENDIF
				ENDDO
			ELSE	!Snow on ice
				melt_frac=MIN(1._dp,seasonablation(d_index)/balance(d_index))
				CALL grainsize((modelday-snowday(d_index)),melt_frac,r_vis,r_nir)
				IF (tmstp .EQ. 86400) THEN
					CALL albedo_compute(solel(12),diffuse_frac,cloud_frac,r_vis,r_nir,a(1))
				ELSE
					CALL albedo_compute(solel(1),diffuse_frac,cloud_frac,r_vis,r_nir,a(1))
				ENDIF
				r_vis=316			!These effective grain sizes are from Arendt(1999)
				r_nir=r_vis			!and were developed for an arctic glacier so may not
									!be the best thing for an alpine glacier
				CALL albedo_compute(solel,diffuse_frac,cloud_frac,r_vis,r_nir,a(2))
			ENDIF						
		ELSE
			IF (balance(d_index) .GT. 0._dp) THEN	!snow albedo with ice lurking below
				melt_frac=MIN(1._dp,seasonablation(d_index)/balance(d_index))
				CALL grainsize((modelday-snowday(d_index)),melt_frac,r_vis,r_nir)
				IF (tmstp .EQ. 86400) THEN
					CALL albedo_compute(solel(12),diffuse_frac,cloud_frac,r_vis,r_nir,a(1))
				ELSE
					CALL albedo_compute(solel(1),diffuse_frac,cloud_frac,r_vis,r_nir,a(1))
				ENDIF
				r_vis=SQRT(3160._dp)
				r_nir=r_vis
				IF (tmstp .EQ. 86400) THEN
					CALL albedo_compute(solel(12),diffuse_frac,cloud_frac,r_vis,r_nir,a(2))
				ELSE
					CALL albedo_compute(solel(1),diffuse_frac,cloud_frac,r_vis,r_nir,a(2))
				ENDIF
			ELSE
				r_vis=SQRT(3160._dp)
				r_nir=r_vis
				IF (tmstp .EQ. 86400) THEN
					CALL albedo_compute(solel(12),diffuse_frac,cloud_frac,r_vis,r_nir,a(1))
				ELSE
					CALL albedo_compute(solel(1),diffuse_frac,cloud_frac,r_vis,r_nir,a(1))
				ENDIF
				a(1)=a(2)
			ENDIF
		ENDIF
		IF (a(1) .LT. a(2)) a(1)=a(2)
		albedo=a(1)+(EXP(-balance(d_index)/scale_depth))*(a(2)-a(1))
!**********************		End grain size dependent albedo model	*********************
!********************************************************************************************

	CASE(3)	!Faron's model
!***	Test to see if the case is snow on snow, snow over ice, or snow over old snow
		IF ((d_index .GT. 2) .OR. ((d_index .EQ. 2) .AND. (balance(1) .GT. 0._dp))) THEN 
!*******	This refers to the case in which snow overlies snow
			top=a_decay*LOG(cum_PDDs(d_index))+a_fresh		!upper layer albedo
			bottom=a_decay*LOG(cum_PDDs((d_index-1)))+a_fresh		!lower layer albedo
			IF (bottom .LT. min_snowalbedo) bottom=min_snowalbedo
			IF (top .LT. min_snowalbedo) top=min_snowalbedo
			IF (bottom .GT. top) bottom=top
			albedo=top+(EXP(-balance(d_index)/scale_depth))*(bottom-top)			
		ELSEIF ((d_index .EQ. 2) .AND. (balance(1) .LE. 0._dp)) THEN
!*******	This is the snow over ice case
			top=a_decay*LOG(cum_PDDs(d_index))+a_fresh
			IF (top .LT. min_snowalbedo) top=min_snowalbedo
			bottom=a_decay_ice*cum_PDDs(1)+a_ice
			IF (bottom .LT. b_albedo) bottom=b_albedo
			albedo=top+(EXP(-balance(d_index)/scale_depth))*(bottom-top)
		ELSEIF (d_index .EQ. 1) THEN
			IF (balance(1) .GT. 0._dp) THEN
!***********	This is the snow over the base albedo case
				top=a_decay*LOG(cum_PDDs(d_index))+a_fresh
				IF (top .LT. min_snowalbedo) top=min_snowalbedo
				bottom=b_albedo
				IF (bottom .GT. top) bottom=top
				albedo=top+(EXP(-balance(d_index)/scale_depth))*(bottom-top)
			ELSE
!***********	This is the ice over base case
				top=a_decay_ice*cum_PDDs(1)+a_ice
				IF (top .LT. b_albedo) top=b_albedo
				bottom=b_albedo
				IF (bottom .GT. top) bottom=top
				albedo=top !+(EXP(-balance(d_index)/scale_depth))*(bottom-top)
			ENDIF
		ELSE
			PRINT *,d_index
			PRINT *,"albedo model error"
			PRINT *,"failure to account for all possible layer scenarios"
			PAUSE
		ENDIF
	END SELECT		
END SUBROUTINE albedo_calc
