SUBROUTINE az_elev(day,hr,latin,longin,az,elev)
USE params
USE nrtype
IMPLICIT NONE
	!  This subroutine calculates solar elevation and azimuth 
	!  for a given latitude and longitude following the subroutine
	!  outlined by R. Walraven(1978).
	!
	!  Inputs:
	!	year 	(as in 19##)
	!	day
	!	hr,min,sec
	!	zone 	(P.S.T.=8)
	!	dasvtm 	(1 if daylight savings, 0 if not)
	!	Lat  	(latitude in positive degrees if north)
	!	long	(longitude in negative degrees if west)
	!	
	!  Outputs:
	!	A (azimuthal angle)
	!	E (solar elevation)
	!
	!*******************************	NOTE	**************************************************
	!
	!  this is the general form of the solar routine used for calculating average solar radiation
	!  so this returns values specific for lattitude only. assuming the length of day is constant
	!  at like latitudes. This modified routine also sets equal to zero the elevation if it is less
	!  than zero.
	!
	!*********************************************************************************************

	REAL (DP),				INTENT(IN   )			:: day,hr,latin,longin
	REAL (DP), 				INTENT(  OUT)			:: az,elev
	REAL (DP)										:: phi
	REAL (DP) 										:: sec=0._dp
	REAL (DP)										:: min=0._dp, &
													   delyr, &
													   t, &
													   theta, &
													   time, &
													   g, &
													   el, &
													   eps, &
													   sel, &
													   ra, &
													   decl, &
													   st, &
													   s, &
													   h

	INTEGER (I4B)									:: leap

	dasvtm=0._dp
	IF ((day .GE. 94) .AND. (day .LE. 300)) dasvtm=1._dp
	delyr=REAL(year)-1980._dp
	leap=IFIX(REAL(delyr)/4.)
	t=hr + (min + sec/60._dp) / 60._dp
	time=delyr*365._dp+leap+day-1._dp+t/24
	IF (delyr .EQ. leap*4. ) time=time-1.
	IF ((delyr .LT. 0.) .AND. (delyr .NE. leap*4.)) time=time-1.
	theta = (360._dp*time/365.25_dp)*rad
	g = -0.031271_dp - 4.53963E-7_dp*time + theta
	el = 4.900968_dp+3.67474E-7_dp*time+(0.033434_dp-2.3E-9_dp*time)*SIN(g)+0.000349_dp*SIN(2._dp*G) +theta
	eps= obliq
	sel=SIN(el)
	ra=ATAN2(sel*COS(eps),COS(el))
	IF (ra .LT. 0.) ra=ra + twopi_d
	decl=ASIN(sel*SIN(eps))
	st=1.759335_dp+twopi_d*(time/365.25_dp-delyr)+3.694E-7_dp*time
	IF (st .GE. twopi_d) st=st-twopi_d
	s=st-longin*rad+1.0027379_dp*(zone-dasvtm+t)*15._dp*rad
	IF (s .GE. twopi_d) s=s-twopi_d
	h = ra - s 
	phi = latin*rad
	elev=ASIN(SIN(phi)*SIN(decl)+COS(phi)*COS(decl)*COS(h))
	IF (ABS(COS(decl)*SIN(h)/COS(elev)) .GT. 1._dp) THEN
		az=90._dp*rad
	ELSE
		az=ASIN(COS(decl)*SIN(h)/COS(elev))
	ENDIF
	IF (SIN(elev) .LT. SIN(decl)/SIN(phi)) THEN
		IF (az .LT. 0) az=az+2._dp*pi
		az=pi-az
	ENDIF
	RETURN
	END	
