PROGRAM ebalcode
USE nrtype
USE params
!USE typeSizes
!USE netcdf
IMPLICIT NONE

!INCLUDE 'netcdf.inc'
!c DEM arrays, Mass balance, Slope and Aspect arrays, sorting arrays

	REAL(DP),	DIMENSION(1:48)				:: hourly_temp,cell_hourly_temp
	REAL(DP),	DIMENSION(1:5)				:: sensible,latent
	REAL(DP),	DIMENSION(1:4)				:: weights
	REAL(DP),	DIMENSION(1:24)				:: solaz,solel,sunup_mask
	REAL(DP),	DIMENSION(80,200)			:: trans_coeff_ice,trans_coeff_snow
	REAL(DP),	DIMENSION(:),	ALLOCATABLE		:: cloud

	REAL(DP),	DIMENSION(:,:), ALLOCATABLE		:: meanhangle, &
								   gdem, &
								   bdem, &
								   S, &
								   A, &
								   balb, &
								   albedo, &
								   t_grid, &
								   rh_grid, &
								   lin_grid, &
								   wind_grid, &
								   balance, &
								   elagrid, &
								   netsw_grid, &
								   ilw_grid, &
								   olw_grid, &
								   hs_grid, &
								   hl_grid, &
								   eflux_grid, &
								   terrainview, &
								   pre_grid, &
								   P_grid, &
								   loadfactor, &
								   g_grid, &
								   ablation, &
								   estor, &
								   latgrid, &
								   longrid, &
								   avp_grid, &
								   svp_grid, &
								   gout_grid, &
								   stemp_grid, &
								   potatm_grid, &
								   pdds_grid, &
								   totablation, &
								   mean_annual_temp, &
								   refreeze, &
								   netloss, &
								   icemask, &
								   giss_topography, &
								   eh_grid, &
								   vpdiffs, &
								   svpg_grid, &
								   rhg_grid, &
								   avpg_grid, &
								   dtg_grid

	REAL*8										:: lin_calc,costheta

	REAL(DP),	DIMENSION(:,:,:),	ALLOCATABLE	:: shading, &
								   massbal, &
								   cum_pdds, &
								   zseasonabl, &
								   zsnowday


	REAL(DP)						:: windy, &
								   weight, &
								   hour, &
								   potential_atm, &
								   doy, &
								   winterbal, &
								   netbal, &
								   diffuse_frac, &
								   elaintercept	!This is a new variable for the linear decrease in ELA with lat


	INTEGER(I4B),	DIMENSION(365,2)				:: month_day_list
	INTEGER(I4B),	DIMENSION(12)					:: mdays
	INTEGER(I4B),	DIMENSION(:,:),	ALLOCATABLE		:: d_index

	INTEGER(I4B) 						:: cellnum=0,&
								   nn, &
								   i, &
								   j, &
								   k, &
								   n, &
								   modelday, &
								   depth_index, &
								   iday, &
								   month, &
								   day, &
								   daycount, &
							   dayidx
!								   status, &
!								   ncbalanceid, &
!								   nclatid, &
!								   nctimeid, &
!								   nclonid, &
!								   ncmassbalid, &
!								   ncnvarsid, &
!								   ncefluxid, &
!								   ncshortwaveid, &
!								   ncalbedoid, &
!								   nclongwaveid, &
!								   ncsensibleid, &
!								   nclatentid, &

	CHARACTER(LEN=15) 					:: filenumber
	CHARACTER(LEN=12)					:: filename
	CHARACTER(LEN=30)					:: invar_filename
	CHARACTER(LEN=10)					:: giss_prefix="giss_laur_"
	CHARACTER(LEN=8)					:: giss_suffix="_100.grd"
	CHARACTER(LEN=2)					:: monthstr,daystr




! Get parameters from the parameter file
	CALL parameters()				!FSA: Modified parameters to be globals, which are declared in
									!the params.f90 file.


! Allocate grid spaces
	ALLOCATE(shading(1:nlat,1:nlong,1:36))				!elevation angle to horizon in 36 azimuth bins
	ALLOCATE(meanhangle(1:nlat,1:nlong))				!menahangle grid of the average angle to horizon
	ALLOCATE(terrainview(1:nlat,1:nlong))				!Fraction of the sky obstructed by terrain
	ALLOCATE(bdem(1:nlat,1:nlong))					!The base elevation grid -- includes the surrounding topography
	ALLOCATE(gdem(1:nlat,1:nlong))					!The elevation grid for the ice only (this could be replaced with an ice mask
	ALLOCATE(icemask(1:nlat,1:nlong))
	ALLOCATE(S(1:nlat,1:nlong))					!Slope grid
	ALLOCATE(A(1:nlat,1:nlong))					!Aspect grid
	ALLOCATE(massbal(1:nlat,1:nlong,1:nlayers))			!Grid storing mass balance at each cell for variety of layers.
	ALLOCATE(balance(1:nlat,1:nlong))				!Total balance grid
	ALLOCATE(balb(1:nlat,1:nlong))					!Background albedo grid
	ALLOCATE(elagrid(1:nlat,1:nlong))
	ALLOCATE(cum_pdds(1:nlat,1:nlong,1:nlayers))			!Total PDDs for the season at all snow depth layers
	ALLOCATE(zsnowday(1:nlat,1:nlong,1:nlayers))			!day of last snowfall
	ALLOCATE(zseasonabl(1:nlat,1:nlong,1:nlayers))			!Total ablation that's occurred ion the season
	ALLOCATE(albedo(1:nlat,1:nlong))				!Surface albedo
	ALLOCATE(t_grid(1:nlat,1:nlong))				!Grid of air temeprature
	ALLOCATE(netsw_grid(1:nlat,1:nlong))				!Grid of net shortwave radiation
	ALLOCATE(ilw_grid(1:nlat,1:nlong))				!grid of incoming longwave radiation
	ALLOCATE(olw_grid(1:nlat,1:nlong))				!grid of outgoing longwave radiation (I think this actually usually stores net longwave
	ALLOCATE(hs_grid(1:nlat,1:nlong))				!grid of sensible heat flux
	ALLOCATE(hl_grid(1:nlat,1:nlong))				!grid of latent heat flux
	ALLOCATE(eflux_grid(1:nlat,1:nlong))				!Grid of the total energy flux to the glacier
	ALLOCATE(d_index(1:nlat,1:nlong))				!grid containing a mapping to which snow depth level a given point's surface is at
	ALLOCATE(cloud(met_length))					!Vector of computed cloud cover for the period of simulation
	ALLOCATE(pre_grid(1:nlat,1:nlong))				!Grid of precipitation rate
	ALLOCATE(P_grid(1:nlat,1:nlong))				!grid of air pressure -- calculated with hydrostatics for the most part
	ALLOCATE(svp_grid(1:nlat,1:nlong))				!grid of saturation vapor pressure
	ALLOCATE(loadfactor(1:nlat,1:nlong))				!Loading factor is a wind redistribution model is applied.
	ALLOCATE(g_grid(1:nlat,1:nlong))				!Solar radiation grid from the met data
	ALLOCATE(ablation(1:nlat,1:nlong))				!grid of ablation for a given time step
	ALLOCATE(estor(1:nlat,1:nlong))					!Stored energy to account for negative nergy balance periods.
	ALLOCATE(latgrid(1:nlat,1:nlong))				!grid of latitude locations
	ALLOCATE(longrid(1:nlat,1:nlong))				!grid of longitude locations
	ALLOCATE(rh_grid(1:nlat,1:nlong))				!relative humidity grid from the input data
	ALLOCATE(lin_grid(1:nlat,1:nlong))				!Grid of incoming longwave radiation from the input met data
	ALLOCATE(wind_grid(1:nlat,1:nlong))				!Grid of incoming wind speeds from the input met data
	ALLOCATE(avp_grid(1:nlat,1:nlong))				!Grid of air vapor pressure calcualted from RH and saturation vapor rpessure
	ALLOCATE(gout_grid(1:nlat,1:nlong))				!Grid of processed solar radiaton
	ALLOCATE(stemp_grid(1:nlat,1:nlong))				!Grid of surface temperature
	ALLOCATE(potatm_grid(1:nlat,1:nlong))				!Grid of potential solar radiation for the regine hock model
	ALLOCATE(pdds_grid(1:nlat,1:nlong))				!Grid of PDDs for a given time step.
	ALLOCATE(totablation(1:nlat,1:nlong))				!Grid of total ablation for a year to be used to calculate refreezing from the MAT.
	ALLOCATE(mean_annual_temp(1:nlat,1:nlong))			!Grid of MAT.
	ALLOCATE(refreeze(1:nlat,1:nlong))				!Grid of max possible refreezing.
	ALLOCATE(netloss(1:nlat,1:nlong))				!Grid of net loss after refreezing.
    ALLOCATE(giss_topography(1:nlat,1:nlong))			!Topography from the GISS model interpolated to the EMBM model resolution -- used to adjust precip and temp
	ALLOCATE(eh_grid(1:nlat,1:nlong))
	ALLOCATE(vpdiffs(1:nlat,1:nlong))
	ALLOCATE(svpg_grid(1:nlat,1:nlong))
	ALLOCATE(rhg_grid(1:nlat,1:nlong))
	ALLOCATE(avpg_grid(1:nlat,1:nlong))
	ALLOCATE(dtg_grid(1:nlat,1:nlong))
	
	totablation=0._dp
	winterbal=0._dp
	!year=2002
	sensible(:)=0._dp
	latent(:)=0._dp
	cum_pdds(:,:,:)=1._dp
	mean_annual_temp(:,:)=0._dp
	balance=0._dp
	WRITE(filenumber,'(i4)') year
	weights=(/0.278,0.5216,0.180,0.0170/)

!***	getdata is a subroutine that reads input data from files and puts them into arrays
!***	for the model to use.
	PRINT *,'going into get data'
	CALL getdata(bdem,gdem,giss_topography,S,A,meanhangle,shading,trans_coeff_snow,trans_coeff_ice,latgrid,longrid)
	PRINT *,'leaving get data'
!***	Open up some of the output files that are written each time step.
	OPEN (UNIT=93,FILE=outputstr//"massbal_misc.txt",STATUS="unknown")

!*********************************************************************************
!*		Compute base albedo for the DEM based on elevation relative to the
!*		AAR determined ELA. This is derived from Oerlemans (1991 or so)
!*********************************************************************************

!$OMP PARALLEL DEFAULT(SHARED)
!$OMP WORKSHARE
	massbal(:,:,:)=0._dp	!blank
	albedo(:,:)=0._dp		!blank
	netsw_grid(:,:)=0._dp	!blank
	ilw_grid(:,:)=0._dp		!blank
	olw_grid(:,:)=0._dp		!blank
	hs_grid(:,:)=0._dp		!blank
	hl_grid(:,:)=0._dp		!blank
	eflux_grid(:,:)=0._dp	!blank
	pre_grid(:,:)=0._dp
	gout_grid(:,:)=0._dp
	stemp_grid(:,:)=0._dp
	balb=0._dp
	elagrid=0._dp
	
	!Going to impliment a latitude dependent ELA for the purpose of keeping the albedo from falling
	!to unrealistically low values in the north of the ice sheet. Currently focusing on a linear implementation
	!Where ELA decreases with latitude. Literature shows ELA decreases at a rate of about 100 m per degree of latitude
	!So, we'll start there and allow the ELA to go to zero in the north
	elaintercept = elastart - 45._dp*elaslope !Here we have chosen the ELA line to go through ELAstart at 45 degrees N
	elagrid(:,:) = latgrid(:,:)*elaslope+elaintercept
	WHERE (elagrid(:,:) .LT. 0._dp)
		elagrid(:,:)=0._dp
	ENDWHERE
	balb(:,:)=a1*ATAN((Bdem(:,:)-elagrid(:,:)+a2)/a3)+a4
	WHERE (balb(:,:) .GT.0.85)
		balb(:,:)=0.85
	ELSEWHERE (balb(:,:) .LT.0.1)
		balb(:,:)=0.10
	ENDWHERE
	albedo(:,:)=balb(:,:)
	massbal(:,:,:)=0._dp
	balance(:,:)=massbal(:,:,1)
	zsnowday(:,:,1)=initday
	terrainview(:,:)=((1._dp+COS(S(:,:)))/2._dp)-meanhangle(:,:)
	WHERE (terrainview(:,:) .LT. 0._dp)
		terrainview(:,:)=0._dp
	ENDWHERE
	meanhangle(:,:)=1._dp-terrainview(:,:)
	P_grid(:,:)=101325.*exp(-Gdem(:,:)/scale_height)
	d_index(:,:)=0
	WHERE (gdem(:,:) .GT. 0._dp)
		d_index(:,:)=1
	ELSEWHERE
		d_index(:,:)=0
	ENDWHERE

	cellnum=SUM(d_index(:,:))
	d_index(:,:)=1
!$OMP END WORKSHARE
!$OMP END PARALLEL
!*********************************************************************************************

!*********************************************************************************************
!Build the index list of days starting 1 Oct and running through 30 Sept to help building of filenames
	mdays=(/31,28,31,30,31,30,31,31,30,31,30,31/)
	daycount=1
	DO month=1,12,1
		DO day=1,mdays(month),1
			IF (daycount .LT. 274) THEN
				dayidx=daycount+(365-273)
			ELSE
				dayidx=daycount-273
			ENDIF
			month_day_list(dayidx,1)=month
			month_day_list(dayidx,2)=day
			daycount=daycount+1
		ENDDO
	ENDDO

!************ Generate fake diurnal cycle. Only for PDD quadrature, so phase doesn't matter **
	IF (tmstp .EQ. 86400) THEN
		DO i=1,48,1
			hourly_temp(i)=diurnal_tamp*SIN(REAL(i)*2._dp*pi/48._dp)
		ENDDO
	ENDIF
!*********************************************************************************************
	WHERE ((gdem(:,:) .LT. 10000) .AND. (gdem .GT. 0.))
		icemask(:,:)=1._dp
	ELSEWHERE
		icemask(:,:)=0._dp
	ENDWHERE

!Old way of doing precip with a two-phase precip lapse rate that double lapses off the giss. Woops!
!	WHERE (gdem(:,:) .LT. 1500._dp)
!		loadfactor(:,:)=(1+gdem(:,:)*pre_laps)
!	ELSEWHERE (gdem(:,:) .GE. 1500._dp)
!		loadfactor(:,:)=((1+1500._dp*pre_laps)-(gdem(:,:)-1500)*pre_laps/2._dp)
!	ENDWHERE

!New way of doing precip
	loadfactor=1+(gdem-giss_topography)*pre_laps

	WHERE (loadfactor(:,:) .LT. 0)
		loadfactor(:,:)=0
	ENDWHERE
	CALL gridder(loadfactor,"pre_enhance.grd")
	filename="meanhgl.grd"
	CALL gridder(meanhangle,TRIM(ADJUSTL(filename)))
	filename="tview.grd"
	CALL gridder(terrainview,TRIM(ADJUSTL(filename)))
	
	
	
	
!***	Now begin the main time loop over the length of the met record.****************
t:	DO nn=1,365,1
!*******	Pulling the input variables from the 4-D meteorological data file
!*******	The t_grid and the pre_grid both have enhancements applied to them
!******		that are set in the parameters for simple sensitivity studies.

!*******	Since the met data starts in jan and we want to start in Oct, need to murk around with
!*******	The day reference to get all on the same page
		iday=nn+274
		IF (iday .GT. 365) iday=iday-365
		doy=REAL(iday)
		IF (doy .EQ. 366.) THEN
			doy=365._dp		!For leap year the 366th day is the same as the 365 in terms of solar (here)
		ENDIF
		modelday=INT(doy)
		hour=0._dp

!$OMP PARALLEL DEFAULT(SHARED)
!$OMP WORKSHARE

		!Have to read in the met data from the file for the day in question
		month=month_day_list(nn,1)
		day=month_day_list(nn,2)
		IF (month .LT. 10) THEN
			WRITE(monthstr,'(i1)') month
		ELSE
			WRITE(monthstr,'(i2)') month
		ENDIF
		IF (day .LT. 10) THEN
			WRITE(daystr,'(i1)') day
		ELSE
			WRITE(daystr,'(i2)') day
		ENDIF
		invar_filename=trim(adjustl(metfilename))//"temp_"//trim(adjustl(monthstr))//"_"//trim(adjustl(daystr)) &
						//"_"//trim(adjustl(resolution))//".grd"
		CALL surf_read(t_grid,invar_filename)
		invar_filename=trim(adjustl(metfilename))//"rh_"//trim(adjustl(monthstr))//"_"//trim(adjustl(daystr)) &
					//"_"//trim(adjustl(resolution))//".grd"
		CALL surf_read(rh_grid,invar_filename)
		invar_filename=trim(adjustl(metfilename))//"insol_"//trim(adjustl(monthstr))//"_"//trim(adjustl(daystr)) &
					//"_"//trim(adjustl(resolution))//".grd"
		CALL surf_read(g_grid,invar_filename)
		invar_filename=trim(adjustl(metfilename))//"wspd_"//trim(adjustl(monthstr))//"_"//trim(adjustl(daystr)) &
					//"_"//trim(adjustl(resolution))//".grd"
		CALL surf_read(wind_grid,invar_filename)
		invar_filename=trim(adjustl(metfilename))//"precip_"//trim(adjustl(monthstr))//"_"//trim(adjustl(daystr)) &
					//"_"//trim(adjustl(resolution))//".grd"
		CALL surf_read(pre_grid,invar_filename)
		invar_filename=trim(adjustl(metfilename))//"svp_"//trim(adjustl(monthstr))//"_"//trim(adjustl(daystr)) &
					//"_"//trim(adjustl(resolution))//".grd"
		CALL surf_read(svpg_grid,invar_filename)
		invar_filename=trim(adjustl(metfilename))//"rhg_"//trim(adjustl(monthstr))//"_"//trim(adjustl(daystr)) &
					//"_"//trim(adjustl(resolution))//".grd"
		CALL surf_read(rhg_grid,invar_filename)


!		mask out the non-ice areas
		t_grid(:,:)=t_grid(:,:)*icemask(:,:)
		rh_grid(:,:)=rh_grid(:,:)*icemask(:,:)
		g_grid(:,:)=g_grid(:,:)*icemask(:,:)
		wind_grid(:,:)=wind_grid(:,:)*icemask(:,:)
		
		
		pre_grid(:,:)=pre_grid*pfact
		pre_grid(:,:)=pre_grid(:,:)*icemask(:,:)
		pre_grid(:,:)=pre_grid(:,:)/1000.*loadfactor		
		t_grid=t_grid+273.16+tdep

!		How the rh and vapour pressures get calculated....
!		Fisrt, calculate the saturation for the input to grid, this is the temp lapsed to sea level from GISS
		svp_grid(:,:)=610.78_dp*exp(17.2693882_dp*(t_grid(:,:)-273.16_dp)/(t_grid(:,:)-35.86_dp))
!		Now lapse up the temperature to the high res topography
		t_grid(:,:)=t_grid(:,:)+laps*gdem(:,:)
	
!		Calculate the air vapor pressure by lapsing the sea-level input to the highres topography.
!		avp_grid=rh_grid*svp_grid*exp(-gdem(:,:)/vaplapse)
!		Calculate a new saturation vapor pressure with the new lapsed temperature
!		svp_grid(:,:)=610.78_dp*exp(17.2693882_dp*(t_grid(:,:)-273.16_dp)/(t_grid(:,:)-35.86_dp))

		
!		Another way of calculating relative humidity....  This involves using the raw, interpolated GISS rh and sat vap 
!		and adjusting only a little based on the topo difference.  This is more in line with the precip methods
		avp_grid=rhg_grid*svpg_grid*exp(-(gdem(:,:)-giss_topography(:,:))/vaplapse)
		dtg_grid=laps*(gdem(:,:)-giss_topography(:,:))
		svp_grid=610.78_dp*exp(17.2693882_dp*((t_grid(:,:)-dtg_grid(:,:))-273.16_dp)/((t_grid(:,:)-dtg_grid(:,:))-35.86_dp))
		rh_grid(:,:)=avp_grid(:,:)/svp_grid(:,:)
		WHERE (rh_grid(:,:) .GT. 1.)
			rh_grid(:,:)=1.
		ENDWHERE
		svp_grid(:,:)=svp_grid(:,:)*icemask(:,:)
		rh_grid(:,:)=rh_grid(:,:)*icemask(:,:)

		pdds_grid(:,:)=0._dp
		eflux_grid(:,:)=0._dp
		netsw_grid(:,:)=0._dp
		olw_grid(:,:)=0._dp
		ilw_grid(:,:)=0._dp
		hs_grid(:,:)=0._dp
		hl_grid(:,:)=0._dp
		vpdiffs(:,:)=0._dp
		ablation(:,:)=0._dp
!***********************************************************************
!$OMP END WORKSHARE
!$OMP END PARALLEL
		diffuse_frac=0.7_dp

!********* Calculate Surface temperature ***********
		WHERE (t_grid(:,:) .GT. 273.16_dp)
			stemp_grid(:,:)=273.16_dp
		ELSEWHERE
			stemp_grid(:,:)=t_grid(:,:)
		ENDWHERE

!***************	Making a big change here: Pulling out stuff from the spatial do loops
!***************	That can be done in parallel. The results will then be sent into the
!***************	next, more difficult to parallelize do loop.
!$OMP PARALLEL DO DEFAULT(SHARED) PRIVATE(i,j,cell_hourly_temp)
longitude1:	DO j=1,nlong
latitude1:	DO i=1,nlat
!			depth_index=d_index(i,j)
			cell_hourly_temp(:)=hourly_temp(:)+t_grid(i,j)-273.16_dp
!***************	Compute PDDs for albedo model  *********************************
!***************	New method in which PDDs are added to all layers on warm days.
!***************	This prevents a dark over white situation, which may or may not be realistic.
!***************	This should eliminate banding in the albedo and pdd arrays though. There will be
!***************	a similar modification in the layer accounting portion of the model.
			IF (tmstp .EQ. 86400) THEN
				IF ((t_grid(i,j)-273.15) .GT. diurnal_tamp) THEN
					cum_pdds(i,j,1:d_index(i,j))=cum_pdds(i,j,1:d_index(i,j))+t_grid(i,j)-273.16
					pdds_grid(i,j)=t_grid(i,j)-273.16
				ELSEIF ((t_grid(i,j)-273.15) .LT. (-1._dp*diurnal_tamp)) THEN
					pdds_grid(i,j)=0._dp
				ELSE
					WHERE (cell_hourly_temp(:) .LT. 0._dp)
						cell_hourly_temp(:)=0._dp
					ENDWHERE
					cum_pdds(i,j,1:d_index(i,j))=cum_pdds(i,j,1:d_index(i,j))+SUM(cell_hourly_temp(:),1)/48._dp
					pdds_grid(i,j)=SUM(cell_hourly_temp(:),1)/48._dp
				ENDIF
			ELSE
				IF ((t_grid(i,j)-273.16) .GT. 0._dp) THEN
					cum_pdds(i,j,1:d_index(i,j))=cum_pdds(i,j,1:d_index(i,j))+(t_grid(i,j)-273.16)*(REAL(tmstp)/86400._dp)
					pdds_grid(i,j)=(t_grid(i,j)-273.16)*(REAL(tmstp)/86400._dp)
				ELSE
					cum_pdds(i,j,1:d_index(i,j))=cum_pdds(i,j,1:d_index(i,j))+0._dp
					pdds_grid(i,j)=0._dp
				ENDIF
			ENDIF
!***************	End new section.....!
!**********************************************************************************************
!determine if precip fell as snow
			IF ((pre_grid(i,j) .GT. 0._dp) .AND. (t_grid(i,j)-tdep .LT. snowthresh)) THEN
				IF ((cum_pdds(i,j,d_index(i,j)) .GE. pdd_thresh) .OR. (massbal(i,j,d_index(i,j)) .LT. 0._dp)) THEN
					d_index(i,j)=d_index(i,j)+1
					cum_pdds(i,j,d_index(i,j))=1._dp
					massbal(i,j,d_index(i,j))=pre_grid(i,j)
					zseasonabl(i,j,d_index(i,j))=0
					zsnowday(i,j,d_index(i,j))=doy
				ELSE
					cum_pdds(i,j,d_index(i,j))=1._dp
					massbal(i,j,d_index(i,j))=massbal(i,j,d_index(i,j))+pre_grid(i,j)
				ENDIF
			ENDIF
			IF (MINVAL(massbal(i,j,2:nlayers)) .LT. 0) THEN
				PRINT *,"layer accounting problem",d_index(i,j),pre_grid(i,j),doy
				PRINT *,massbal(i,j,:)
				STOP
			ENDIF
		END DO	latitude1
		END DO	longitude1

!$OMP END PARALLEL DO







longitude:	DO j=1,nlong
latitude:	DO i=1,nlat
!***********	check to see if the current indicies are over the glacier if not, skip the step
			IF (gdem(i,j) .GT. 0._dp) THEN
				depth_index=d_index(i,j)
				cell_hourly_temp(:)=hourly_temp(:)+t_grid(i,j)-273.16_dp
				IF (tmstp .EQ. 86400) THEN
					DO n=1,24,1
						hour=REAL(n-1)
						CALL az_elev(doy,hour,latgrid(i,j),longrid(i,j),solaz(n),solel(n))
					ENDDO
				ELSE
					CALL az_elev(doy,hour,latgrid(i,j),longrid(i,j),solaz(1),solel(1))
				ENDIF
				sunup_mask(:)=0._dp
				WHERE (solel(:) .GT. 0._dp)
					sunup_mask(:)=1._dp
				ENDWHERE

Model_selector:	SELECT CASE (model_type)

				CASE(1)
!*******************	Anslow Model
					CALL solarcalc(s(i,j),a(i,j),doy,solaz,solel,meanhangle(i,j),terrainview(i,j), &
						shading(i,j,1:36),P_grid(i,j),g_grid(i,j),potential_atm,gout_grid(i,j))
					ablation(i,j)=(t_grid(i,j)-273.16)*temp_coef*tmstp/86400._dp &
						+g_grid(i,j)*(1-albedo(i,j))*tmstp*rad_coef/1000._dp
					IF (ablation(i,j) .LT. 0._dp) ablation(i,j)=0._dp
					zseasonabl(i,j,depth_index)=zseasonabl(i,j,depth_index)+ablation(i,j)
					massbal(i,j,depth_index)=massbal(i,j,depth_index)-ablation(i,j)
				CASE(2)
!*******************	Hock Model
					CALL solarcalc(s(i,j),a(i,j),doy,solaz,solel,meanhangle(i,j),terrainview(i,j), &
						shading(i,j,1:36),P_grid(i,j),g_grid(i,j),potential_atm,gout_grid(i,j))
					IF (pdds_grid(i,j) .GT. 0._dp) THEN  !melt occurs
						IF (depth_index .GT. 1) THEN !There should be some snow around, use hock_snow
							ablation(i,j)=pdds_grid(i,j)*(meltfactor+hock_snow*potential_atm)
						ELSEIF (massbal(i,j,depth_index) .GT. 0._dp) THEN !Also snow
							ablation(i,j)=pdds_grid(i,j)*(meltfactor+hock_snow*potential_atm)
						ELSE							!Ice is the only remaining option
							ablation(i,j)=pdds_grid(i,j)*(meltfactor+hock_ice*potential_atm)
						ENDIF
					ELSE				!nada
						ablation(i,j)=0._dp
					ENDIF
					zseasonabl(i,j,depth_index)=zseasonabl(i,j,depth_index)+ablation(i,j)
					massbal(i,j,depth_index)=massbal(i,j,depth_index)-ablation(i,j)
				CASE (3)
!*******************	PDD Model
					IF (pdds_grid(i,j) .GT. 0._dp) THEN  !melt occurs
						IF (depth_index .GT. 1) THEN !There should be some snow around, use DDF_snow
							ablation(i,j)=pdds_grid(i,j)*ddf_snow
						ELSEIF (massbal(i,j,depth_index) .GT. 0._dp) THEN !Also snow
							ablation(i,j)=pdds_grid(i,j)*ddf_snow
						ELSE							!Ice is the only remaining option
							ablation(i,j)=pdds_grid(i,j)*ddf_ice
						ENDIF
					ELSE				!nada
						ablation(i,j)=0._dp
					ENDIF
					zseasonabl(i,j,depth_index)=zseasonabl(i,j,depth_index)+ablation(i,j)
					massbal(i,j,depth_index)=massbal(i,j,depth_index)-ablation(i,j)
				CASE (4)
!*******************	Energy Balance Model

					!CALL solarcalc(s(i,j),a(i,j),doy,solaz,solel,meanhangle(i,j),terrainview(i,j), &
					!	shading(i,j,1:36),P_grid(i,j),g_grid(i,j),potential_atm,gout_grid(i,j))
					!Short circuiting the solarcalc routine by injecting measured solar into the gout grid
					WHERE (solel(:) .LT. 0._dp)
						solel(:)=0
					ENDWHERE
						IF ((i .EQ. size(latgrid,1)) .AND. (j .EQ. 1)) THEN
						WRITE(UNIT=6,FMT='(5(f10.2,1x))') latgrid(i,j),doy,sum(sunup_mask),maxval(solel),costheta(S(i,j),A(i,j),0._dp,maxval(solel))
					ENDIF
					!PAUSE
					gout_grid(i,j)=g_grid(i,j)*costheta(S(i,j),A(i,j),0._dp,maxval(solel)) !(sum(solel)/sum(sunup_mask)))
					potential_atm=gout_grid(i,j)
					!End short circuit

					lin_grid(i,j)=lin_calc(rh_grid(i,j),t_grid(i,j))
					CALL lwcalc(modelday,t_grid(i,j),tdep,meanhangle(i,j),terrainview(i,j), &
						avp_grid(i,j),lin_grid(i,j),cloud, &
						stemp_grid(i,j),olw_grid(i,j),ilw_grid(i,j))
!*******************	Check to see if statwind option is applied. If so, apply quadrature on sensible
!*******************	and latent heat fluxes to account for gustiness in the wind field.
					IF (statwind) THEN
						DO k=1,4,1
							windy=wind_grid(i,j)*(1._dp/3._dp)*(2*REAL(k-1)+1)
							weight=weights(k)
							IF (windy .LT. 1.0_dp) THEN
								sensible(k)=0._dp
								latent(k)=0._dp
							ELSE
								CALL turbcalc(avp_grid(i,j),svp_grid(i,j),t_grid(i,j),windy, &
									P_grid(i,j),massbal(i,j,depth_index), &
									stemp_grid(i,j),sensible(k),latent(k),trans_coeff_snow, &
									trans_coeff_ice,eh_grid(i,j),vpdiffs(i,j))
								sensible(k)=sensible(k)*weight
								latent(k)=latent(k)*weight
							ENDIF
						ENDDO
						hs_grid(i,j)=SUM(sensible(:))
						hl_grid(i,j)=SUM(latent(:))
						sensible(:)=0._dp
						latent(:)=0._dp
					ELSE
						CALL turbcalc(avp_grid(i,j),svp_grid(i,j),t_grid(i,j),wind_grid(i,j), &
							P_grid(i,j),massbal(i,j,depth_index), &
							stemp_grid(i,j),hs_grid(i,j),hl_grid(i,j), &
							trans_coeff_snow,trans_coeff_ice,eh_grid(i,j),vpdiffs(i,j))
					ENDIF
					netsw_grid(i,j)=(1._dp-albedo(i,j))*gout_grid(i,j)
					eflux_grid(i,j)=netsw_grid(i,j)+olw_grid(i,j)+hs_grid(i,j)+hl_grid(i,j)
					estor(i,j)=0._dp
					IF (eflux_grid(i,j) .GT. 0._dp) THEN
						IF (eflux_grid(i,j) .GE. ABS(estor(i,j))) THEN
							eflux_grid(i,j)=eflux_grid(i,j)+estor(i,j)
							ablation(i,j)=eflux_grid(i,j)*REAL(tmstp)/lf/rhowat
							estor(i,j)=0._dp
							IF (hl_grid(i,j) .LT. 0._dp) ablation(i,j)=ablation(i,j) -tmstp*hl_grid(i,j)/ls/1000.
						ELSE
						!	estor(i,j)=eflux_grid(i,j)+estor(i,j)
							ablation(i,j)=0._dp
						ENDIF

					ELSE
					!	estor(i,j)=estor(i,j)+eflux_grid(i,j)*0.4_dp
						ablation(i,j)=0._dp
					ENDIF
					zseasonabl(i,j,depth_index)=zseasonabl(i,j,depth_index)+ablation(i,j)
					massbal(i,j,depth_index)=massbal(i,j,depth_index)-ablation(i,j)
				END SELECT Model_selector
!***************	Now we need to account for loss of layers with the ablation that has occurred during this time step
				CALL layeraccount(massbal,cum_pdds,zseasonabl,zsnowday,depth_index,i,j)
!***************	Now it's time to calculate a new albedo for the Anslow and energy balance models
Albedo_selector:		SELECT CASE (model_type)
				CASE(1,4)
					CALL albedo_calc(massbal(i,j,:),cum_PDDs(i,j,:),depth_index,balb(i,j), &
						zseasonabl(i,j,:),zsnowday(i,j,:),doy,solel,diffuse_frac,albedo(i,j))
				CASE DEFAULT
					CONTINUE
				END SELECT Albedo_selector
!*************************************************************************************************


!***************	Reset variables
				d_index(i,j)=depth_index
			END IF
		    END DO	latitude
        	END DO	longitude

		WHERE (ablation(:,:) .GT. 0)
			totablation(:,:)=ablation(:,:)+totablation(:,:)
		ENDWHERE
		
		balance(:,:)=SUM(massbal(:,:,:),3)
		netbal=(SUM(balance))/cellnum
		WRITE(UNIT=93,FMT='(f10.3,1x,6(f10.3,1x))') netbal, &
		SUM(netsw_grid)/cellnum,SUM(olw_grid)/cellnum,SUM(hs_grid)/cellnum, &
		SUM(hl_grid)/cellnum,SUM(pre_grid)/cellnum,SUM(eflux_grid)/cellnum
		WRITE(filenumber,'(i3)') nn
		IF (diagnostic) THEN
			filename="temp"//TRIM(ADJUSTL(filenumber))//".grd"
			CALL gridder(t_grid,TRIM(ADJUSTL(filename)))
			filename="netsw"//TRIM(ADJUSTL(filenumber))//".grd"
			CALL gridder(netsw_grid,TRIM(ADJUSTL(filename)))
			filename="abdo"//TRIM(ADJUSTL(filenumber))//".grd"
			CALL gridder(albedo,TRIM(ADJUSTL(filename)))
			filename="lin"//TRIM(ADJUSTL(filenumber))//".grd"
			CALL gridder(lin_grid,TRIM(ADJUSTL(filename)))
			filename="olw"//TRIM(ADJUSTL(filenumber))//".grd"
			CALL gridder(olw_grid,TRIM(ADJUSTL(filename)))
			filename="hl"//TRIM(ADJUSTL(filenumber))//".grd"
			CALL gridder(hl_grid,TRIM(ADJUSTL(filename)))
			filename="hs"//TRIM(ADJUSTL(filenumber))//".grd"
			CALL gridder(hs_grid,TRIM(ADJUSTL(filename)))
			filename="bal"//TRIM(ADJUSTL(filenumber))//".grd"
			CALL gridder(balance,TRIM(ADJUSTL(filename)))
			filename="eflx"//TRIM(ADJUSTL(filenumber))//".grd"
			CALL gridder(eflux_grid,TRIM(ADJUSTL(filename)))
			filename="rh"//TRIM(ADJUSTL(filenumber))//".grd"
			CALL gridder(rh_grid,TRIM(ADJUSTL(filename)))
			filename="eh"//TRIM(ADJUSTL(filenumber))//".grd"
			CALL gridder(eh_grid,TRIM(ADJUSTL(filename)))
			filename="wind"//TRIM(ADJUSTL(filenumber))//".grd"
			CALL gridder(wind_grid,TRIM(ADJUSTL(filename)))
			filename="gout"//TRIM(ADJUSTL(filenumber))//".grd"
			CALL gridder(gout_grid,TRIM(ADJUSTL(filename)))
		END IF

		IF (netbal .GT. winterbal) winterbal=netbal
		mean_annual_temp(:,:)=mean_annual_temp(:,:)+(t_grid(:,:))/365.
	END DO	t
	!At the end of the mass balance year, need to adjust the net balance for refreezing.
	!We do this by adjusting the mass balance by any of the ablations that may have
	!refrozen in the snow or frozen as superimposed ice on or within the ice sheet. We do
	!this following Huybrechts and De Woul, 1999
	WHERE (mean_annual_temp .LT. 273.16)
		refreeze(:,:)=((273.16-mean_annual_temp)*2.*2009./335000.)
	ELSEWHERE
		refreeze(:,:)=0._dp
	ENDWHERE
	CALL gridder(refreeze*icemask,"total_refreezing.grd")
	CALL gridder(totablation*icemask,"total_ablation.grd")
	netloss(:,:)=totablation(:,:)-refreeze(:,:);
	WHERE (netloss(:,:) .LT. 0._dp) netloss(:,:)=0._dp
	balance=balance(:,:)+(totablation(:,:)-netloss(:,:))
	balance=balance*icemask

	! Re calculating the end of year mass balanc after refreezing has been accounted for.
	! Writing to Output/massbal_misc.txt as the last line.
	netbal=(SUM(balance))/cellnum
	WRITE(UNIT=93,FMT='(f10.3,1x,6(f10.3,1x))') netbal, &
	SUM(netsw_grid)/cellnum,SUM(olw_grid)/cellnum,SUM(hs_grid)/cellnum, &
	SUM(hl_grid)/cellnum,SUM(pre_grid)/cellnum,SUM(eflux_grid)/cellnum

	CLOSE (UNIT=92)
	CLOSE (UNIT=93)
	CLOSE (UNIT=94)
	PRINT *,'calling gridder for write of mass balance grid'
	CALL gridder(balance(:,:),'eoy_balance.grd')
	DO i=1,numstk
	   CLOSE(UNIT=(30+i))
	ENDDO
END PROGRAM ebalcode



