REAL*8 FUNCTION lin_calc(relativehumidity,temperature)
USE params
USE nrtype
IMPLICIT NONE

!************************************************************************************
!		This subroutine is designed to calculate incoming longwave for the
!		atmosphere given a set of humidity conditions. Presently, this is just using
!		the longwave parameterization developed for South Cascade glacier for 2004. This
!		parameterization uses relative humidity to calculate an effective emissivity
!		with a 6th order polynomial with coefficients given in the variable
!		lincalc_coeffs. This is coupled with temperature in the stephan boltzman BB law.
!****************************************************************************************

REAL(DP),					INTENT(IN   )	:: relativehumidity, &
											   temperature
REAL(DP)									:: calculated_emiss,rh
REAL(DP),	DIMENSION(7)					:: lincalc_coeffs

INTEGER(I4B)								:: k

	rh=0._dp
	calculated_emiss=0._dp
	lincalc_coeffs=(/-1.8687285e-11,5.2013591e-9,-5.3018022e-7,2.4046946e-5, &
					-3.9569082e-4,-2.1312186e-3,8.2970495e-1/)
	!lincalc_coeffs=(/0.,0.,0.,4.22632e-7, &
	!				-2.5051e-5,2.230631e-3,6.56963526e-1/)
	rh=relativehumidity
	IF (rh .GT. 1._dp) rh=1._dp
	DO k=1,6
		calculated_emiss=calculated_emiss+lincalc_coeffs(k)*rh**(REAL(7-k))
	ENDDO
	calculated_emiss=calculated_emiss+lincalc_coeffs(7)
	lin_calc=calculated_emiss*stef_bolz*(temperature**4)


END FUNCTION lin_calc



