SUBROUTINE parameters()
USE nrtype
USE params
IMPLICIT NONE
!c
!***********************************************************************
!* Abstract:  collects various geographic and climatic variables needed*
!*		  for specific runs from "parameters" file.		  *
!*											   *
!* Input: Data is returned to the main program in the following order: *
!*											   *
!*	  model  number used to choose which set of ebalance equations *
!*		   to use: 1=Oerlemans(1992), 2=Paterson(1994),	    *
!*		   3=Arnold, et.al.(1996).					 *
!*	  belev  base elevation for grid (meters).			   *
!*	  Khs    turbulent heat bulk transfer coefficient (for snow    *
!*		   if using Paterson"s equations, for snow and ice if    *
!*		   using Oerleman"s).						*
!*	  Khi    turbulent heat bulk transfer coefficient for ice	*
!*		   (Paterson).							 *
!*	  Tdep   temperature depression (degrees).			   *
!*	  a1-a6  albedo parameterizations.				     *
!*	  ELA    initial E.L.A. of reconstructed glacier (meters).     *
!*	  zone   time zone based on GUT (Pacific St. Time = zone 8).   *
!*	  lat    local latitude of Gdem in degrees north (e.g. 35.783). *
!*	  lon    local longitude of Gdem in degrees west of greenwich.  *
!*	  dasvtm toggle for daylight savings time (1=on, 0=off).	 *
!*	  status "old" or "new": tells program if a run has been made  *
!*		     for this particular glacier and timespan before.    *
!*		     if "old" then the model skips the solar and terrain *
!*		     subroutines to save time.				   *
!*	  spack* variables to compute snowpack at start of model run.  *
!*	  stkloc integer array to contain grid points at which energy  *
!*		     balance data will be recorded.			    *
!*	  numstk number of stake locations (data files).		   *
!*	  tmstp  number of time steps per day to calculate energy	*
!*		     balace variables.					     *
!*	  snowdns snow density for use in spack equation! added 1-29-99*
!*	  pre_laps variable for precipitation lapse rate calc.	 *
!*		     4-9-99							    *
!***********************************************************************
!c
!c Declarations
	REAL(DP)						:: ndays
	INTEGER(I4B) 					:: i,round,ios
	CHARACTER(len=4)						:: trash
!cswh	CHARACTER*3 status
!c
!c Open parameters file for model variables
	OPEN (UNIT=10, file=trim(adjustl(inputstr))//"param2d.txt", status="old",iostat=ios)

	READ(UNIT=10,FMT=*) resolution

	!From the resolution, derive the lats and lons filenames:
	READ(UNIT=10,FMT=*) basegridname
	slopegridname=trim(adjustl(basegridname))//"slope_"//trim(adjustl(resolution))//".grd"
	aspectgridname=trim(adjustl(basegridname))//"aspect_"//trim(adjustl(resolution))//".grd"
	meanhanglegridname=trim(adjustl(basegridname))//"meanhangle_"//trim(adjustl(resolution))//".grd"
	shadingfilename=trim(adjustl(basegridname))//"shading_"//trim(adjustl(resolution))//".dat"
	latsfilename=trim(adjustl(basegridname))//"lats_"//trim(adjustl(resolution))//".grd"
	lonsfilename=trim(adjustl(basegridname))//"lons_"//trim(adjustl(resolution))//".grd"


	blankedgridname=trim(adjustl(basegridname))//trim(adjustl(resolution))//"_base.grd"
	basegridname=trim(adjustl(basegridname))//trim(adjustl(resolution))//".grd"
	gisstoponame="giss_topo_"//trim(adjustl(resolution))//".grd"
	!Open the base grid and extract out the nlong and nlat
	OPEN (UNIT=11,file=inputstr//trim(adjustl(basegridname)))
	READ(UNIT=11,FMT='(a4)') trash
	READ(UNIT=11,FMT=*) nlong,nlat
	CLOSE(UNIT=11)

	READ(UNIT=10,FMT=*) metfilename
	READ(UNIT=10,FMT=*) stakefilename
	READ(UNIT=10,FMT=*) balancefilename
	READ(UNIT=10,FMT=*) diagnostic
	READ(UNIT=10,FMT=*) model
	READ(UNIT=10,FMT=*) albedo_model
	READ(UNIT=10,FMT=*) Belev
	READ(UNIT=10,FMT=*) pre_elev
	READ(UNIT=10,FMT=*) rhelev
	READ(UNIT=10,FMT=*) Khs
	READ(UNIT=10,FMT=*) Khi
	READ(UNIT=10,FMT=*) Tdep
	READ(UNIT=10,FMT=*) pfact
	READ(UNIT=10,FMT=*) snowthresh
	READ(UNIT=10,FMT=*) diurnal_tamp
	READ(UNIT=10,FMT=*) snow_coeff1
	READ(UNIT=10,FMT=*) snow_coeff2
	READ(UNIT=10,FMT=*) snow_coeff3
	READ(UNIT=10,FMT=*) snow_coeff4
	READ(UNIT=10,FMT=*) snowinit
	READ(UNIT=10,FMT=*) pdd_thresh
	READ(UNIT=10,FMT=*) zone
	READ(UNIT=10,FMT=*) lat
	READ(UNIT=10,FMT=*) lon
	READ(UNIT=10,FMT=*) obliq
	READ(UNIT=10,FMT=*) eccentricity
	READ(UNIT=10,FMT=*) precession
	READ(UNIT=10,FMT=*) excel_date
	READ(UNIT=10,FMT=*) numstk
	IF (numstk .GT. 0) THEN
		OPEN(UNIT=11,FILE=inputstr//'stk_locs.txt',STATUS='old')
		DO i=1,numstk
			READ(UNIT=11,FMT=*) stkloc(i,1),stkloc(i,2)
		ENDDO
		CLOSE(UNIT=11)
	ENDIF
	READ(UNIT=10,FMT=*) numouts
	IF (numouts .GT. 0) THEN
		OPEN(UNIT=11,FILE=inputstr//'outdays.txt',STATUS='old')
		DO i=1,numouts
			READ(UNIT=11,FMT=*) out_days(i)
		ENDDO
		CLOSE(UNIT=11)
	ENDIF
	READ(UNIT=10,FMT=*) tmstp
	daily_steps=round(86400/tmstp)
	READ(UNIT=10,FMT=*) snowdns
	READ(UNIT=10,FMT=*) ninsts
	READ(UNIT=10,FMT=*) met_length
	READ(UNIT=10,FMT=*) nlayers
	READ(UNIT=10,FMT=*) turbflux_tol
	READ(UNIT=10,FMT=*) inst_height
	READ(UNIT=10,FMT=*) statwind
	READ(UNIT=10,FMT=*) tippingbucket
	READ(UNIT=10,FMT=*) ddf_snow
	READ(UNIT=10,FMT=*) ddf_ice
	READ(UNIT=10,FMT=*) hock_snow
	READ(UNIT=10,FMT=*) hock_ice
	READ(UNIT=10,FMT=*) meltfactor
	READ(UNIT=10,FMT=*) temp_coef
	READ(UNIT=10,FMT=*) rad_coef
	READ(UNIT=10,FMT=*) model_type
	READ(UNIT=10,FMT=*) t2elev
	READ(UNIT=10,FMT=*) t3elev
	READ(UNIT=10,FMT=*) year
	READ(UNIT=10,FMT=*) wangle
	READ(UNIT=10,FMT=*) a1
	READ(UNIT=10,FMT=*) a2
	READ(UNIT=10,FMT=*) a3
	READ(UNIT=10,FMT=*) a4
	READ(UNIT=10,FMT=*) a5
	READ(UNIT=10,FMT=*) a6
	READ(UNIT=10,FMT=*) a7
	CLOSE(UNIT=10)

	!Now get the sensitivity parameters
	OPEN(UNIT=10,file=inputstr//"sensitivity_params.txt",STATUS="unknown")
	READ(UNIT=10,FMT=*) zo_ice
	READ(UNIT=10,FMT=*) zo_snow
	READ(UNIT=10,FMT=*) laps
	READ(UNIT=10,FMT=*) pre_laps
	READ(UNIT=10,FMT=*) vaplapse
	READ(UNIT=10,FMT=*) trans
	READ(UNIT=10,FMT=*) terrain_albedo
	READ(UNIT=10,FMT=*) a_decay
	READ(UNIT=10,FMT=*) a_decay_ice
	READ(UNIT=10,FMT=*) min_snowalbedo
	READ(UNIT=10,FMT=*) a_fresh
	READ(UNIT=10,FMT=*) a_ice
	READ(UNIT=10,FMT=*) elaslope
	READ(UNIT=10,FMT=*) elastart
	CLOSE (UNIT=10)
	PRINT *,"laps: ",laps
	PRINT *,"pre_laps: ",pre_laps
	PRINT *,"zo_ice: ",zo_ice
	PRINT *,"zo_snow: ",zo_snow
	PRINT *,"trans: ",trans
	PRINT *,"terrain_albedo: ",terrain_albedo
	PRINT *,"a_decay: ",a_decay
	PRINT *,"a_decay_ice: ",a_decay_ice
	PRINT *,"ELA at 45 deg: ",elastart
	PRINT *,"ELA slope: ",elaslope

	ndays=met_length*(tmstp/86400.)
	IF (ndays .GT. 365) THEN
		longrunstat=.TRUE.
	ELSE
		longrunstat=.FALSE.
	ENDIF
	RETURN
	END
