MODULE	params
USE nrtype
IMPLICIT NONE


!	This module declares variables that are used everywhere in the
!	model. The list will surely grow. It also contains constants that
!	can be declared as parameters here and then forgotten about
!	later in the model. These do not need to be changed unless the
!	sun goes red giant in which case Solconst probably should be adjusted.



	REAL(DP)				:: Khs=0._dp
	REAL(DP)				:: khi=0._dp
	REAL(DP)				:: laps=0.0_dp	 	! Dry adiabatic lapse rate: deg/m
	REAL(DP)				:: snowthresh=0._dp		!snowfall temperature threshold.
	REAL(DP)				:: diurnal_tamp=5._dp		!Diurnal T amplitude for sine function
	REAL(DP)				:: a1=0._dp
	REAL(DP)				:: a2=0._dp
	REAL(DP)				:: a3=0._dp
	REAL(DP)				:: a4=0._dp
	REAL(DP)				:: a5=0._dp
	REAL(DP)				:: a6=0._dp
	REAL(DP)				:: a7=0._dp
	REAL(DP)				:: min_snowalbedo=0._dp
	REAL(DP)				:: a_fresh=0._dp		!fresh snow albedo
	REAL(DP)				:: a_ice=0._dp		!freshly uncovered ice albedo
	REAL(DP)				:: a_decay=-0._dp		!logarythmic decay rate of snow albedo with temperature
	REAL(DP)				:: a_decay_ice=-0._dp
	!These are the parameters for the Arendt snow and ice albedo model
	REAL(DP)				:: r_o=200._dp
	REAL(DP)				:: r_max=1600._dp
	REAL(DP)				:: t_max=9._dp
	REAL(DP)				:: kappa_vis=6.8620E-3_dp !4.4E-3_dp	!2.0E-3._dp
	REAL(DP)				:: kappa_nir=7.284E-3_dp !13.32E-3_dp	!2.123E-3_dp
	REAL(DP)				:: kappa_vis_theta=4.718E-3_dp !4.4E-3_dp	!1.375E-3_dp
	REAL(DP)				:: kappa_nir_theta=6.8627E-3_dp !13.32E-3_dp	!2.0E-3_dp
	REAL(DP)				:: a_nir_max=0.85
	REAL(DP)				:: a_vis_max=1.00
	!End arendt parameters
	REAL(DP)				:: elaslope=0._dp
	REAL(DP)				:: elastart=0._dp
	REAL(DP)				:: snow_coeff1=0._dp
	REAL(DP)				:: snow_coeff2=0._dp
	REAL(DP)				:: snow_coeff3=0._dp
	REAL(DP)				:: snow_coeff4=0._dp
	REAL(DP)				:: pdd_thresh=0._dp
	REAL(DP)				:: lat=0._dp
	REAL(DP)				:: lon=0._dp
	REAL(DP)				:: excel_date=0._dp
	REAL(DP)				:: Tdep=0._dp
	REAL(DP)				:: pfact=1._dp
	REAL(DP)				:: belev=0._dp
	REAL(DP)				:: pre_elev=0._dp
	REAL(DP)				:: rhelev=0._dp
	REAL(DP)				:: pre_laps=0._dp
	REAL(DP)				:: snowdns=0._dp
	REAL(DP)				:: cell_size=0._dp
	REAL(DP)				:: zo_ice=0._dp
	REAL(DP)				:: zo_snow=0._dp
	REAL(DP)				:: turbflux_tol=0._dp
	REAL(DP)				:: inst_height=0._dp
	REAL(DP)				:: trans=0._dp
	REAL(DP)				:: terrain_albedo=0._dp
	REAL(DP)				:: snowinit=0._dp
	REAL(DP)				:: initday=0._dp
	REAL(DP)				:: temp_coef=0._dp
	REAL(DP)				:: rad_coef=0._dp
	REAL(DP)				:: meltfactor=0._dp
	REAL(DP)				:: hock_snow=0._dp
	REAL(DP)				:: hock_ice=0._dp
	REAL(DP)				:: ddf_snow=0._dp
	REAL(DP)				:: ddf_ice=0._dp
	REAL(DP)				:: obliq=0._dp
	REAL(DP)				:: eccentricity=0._dp
	REAL(DP)				:: precession=0._dp
	REAL(DP)				:: xmin,xmax,ymin,ymax
	REAL(DP)				:: t2elev
	REAL(DP)				:: t3elev
	REAL(DP)				:: wangle


	REAL(DP),	DIMENSION(1:96)		:: out_days=0._dp

	REAL(DP),	PARAMETER		:: gasconst_ratio=0.622_dp	!This is Rd/Rv in Wallace and Hobbs
	REAL(DP),	PARAMETER		:: vonkarman=0.4_dp
	REAL(DP),	PARAMETER		:: sea_density=1.29		! Density of air at STP
	REAL(DP),	PARAMETER		:: sea_press=101324.6_dp	! Standard SLP.
	REAL(DP),	PARAMETER		:: Lf=334000._dp	  	! Latent heat of fusion of water (J/kg)
	REAL(DP),	PARAMETER		:: Lv=2514000._dp
	REAL(DP),	PARAMETER		:: Ls=2849000._dp
	REAL(DP),	PARAMETER		:: Cp=2106._dp			! Specific heat of ice:  (J/deg/kg)
	REAL(DP),	PARAMETER		:: cp_air=1008.7_dp		! Specific heat capacity of moist air
	REAL(DP),	PARAMETER		:: rhowat=1000._dp	 	! Density of water: kg/m^3
	REAL(DP),	PARAMETER		:: Solconst=1368._dp	   	! solar constant:  W/m^2, adj. 3-1-99
	REAL(DP),	PARAMETER		:: rad=0.01745329251994_dp	! degrees to radians conversion
	REAL(DP),	PARAMETER		:: stef_bolz=5.67051e-8_dp 	!stefan-bolzman constant
	REAL(DP),	PARAMETER		:: r_h=0.79714679_dp		! ratio of earth radius to atmospheric height
	REAL(DP),	PARAMETER		:: blank=-9999_dp		!surfer blank value
	REAL(DP),	PARAMETER		:: earthrad=6.37814E6_dp	!Radius of the earth
	REAL(DP),	PARAMETER		:: ice_emiss=0.988_dp		!emissivity of ice
	REAL(DP),	PARAMETER		:: scale_depth=0.025_dp		!this is the scale depth for snow transparency between layers for albedo calculation
	REAL(DP),	PARAMETER		:: maxelevation=9999._dp
	REAL(DP),	PARAMETER		:: minelevation=-1._dp
	REAL(DP),	PARAMETER		:: gravity=9.80665_dp
	REAL(DP),	PARAMETER		:: maxcloudthresh=0.8_dp
	REAL(DP),	PARAMETER		:: mincloudthresh=0.4_dp
	REAL(DP),	PARAMETER		:: scale_height=8100._dp	!atmospheric scale height
	REAL(DP),	PARAMETER		:: pts=10._dp			!number of data points to smooth for SR50
	REAL(DP)					:: vaplapse=4500._dp

	INTEGER(I4B),	DIMENSION(30,2)		:: stkloc

	INTEGER(I4B)				:: zone=0
	INTEGER(I4B)				:: model=0
	INTEGER(I4B)				:: albedo_model=0
	INTEGER(I4B) 			 	:: tmstp=0
	INTEGER(I4B)				:: dasvtm=0
	INTEGER(I4B)				:: numstk=0
	INTEGER(I4B)				:: numouts=0
	INTEGER(I4B)				:: nlat=0
	INTEGER(I4B)				:: nlong=0
	INTEGER(I4B)				:: ninsts=0
	INTEGER(I4B)				:: nlayers=0
	INTEGER(I4B)				:: met_length=0
	INTEGER(I4B)				:: cloudy_q
	INTEGER(I4B)				:: runnumber=0
	INTEGER(I4B)				:: daily_steps
	INTEGER(I4B)				:: model_type
	INTEGER(I4B)				:: year

	INTEGER(I4B),	PARAMETER		:: HC=36	! max number of columns in horizon array
	INTEGER(I4B),	PARAMETER		:: maxiterations=20

	CHARACTER(LEN=3)			:: resolution=""
	CHARACTER(LEN=5)			:: coordtype="ortho"
	CHARACTER(LEN=6)			:: inputstr="Input/"
	CHARACTER(LEN=7)			:: outputstr="Output/"
	CHARACTER(LEN=30)			:: basegridname
	CHARACTER(LEN=30)			:: blankedgridname
	CHARACTER(LEN=30)			:: gisstoponame
	CHARACTER(LEN=30)			:: slopegridname
	CHARACTER(LEN=30)			:: aspectgridname
	CHARACTER(LEN=30)			:: metfilename
	CHARACTER(LEN=30)			:: meanhanglegridname
	CHARACTER(LEN=30)			:: shadingfilename
	CHARACTER(LEN=30)			:: solazfilename
	CHARACTER(LEN=30)			:: solelfilename
	CHARACTER(LEN=30)			:: stakefilename
	CHARACTER(LEN=30)			:: balancefilename
	CHARACTER(LEN=30)			:: latsfilename
	CHARACTER(LEN=30)			:: lonsfilename

	LOGICAL					:: statwind, &
						   tippingbucket, &
						   sw_stat=.TRUE., &
						   lin_stat=.TRUE., &
						   t2_stat=.TRUE., &
						   t3_stat=.TRUE., &
						   daily_balance=.TRUE., &
						   longrunstat=.FALSE., &
						   hut_stat=.TRUE., &
						   laps_stat=.TRUE., &
						   diagnostic=.FALSE.

END MODULE params
