SUBROUTINE shady(hsurf,hsurf_glac,shading)
USE nrtype
USE params
IMPLICIT NONE

!***************************************************************************************************
!*
!*	This program generates lookup tables for generating shading informateion for a given DEM.
!*	More specifically, this subroutine will return an array of size nlong x ncolumns x nbins
!*	Where nbins are the number of azimuth bins for which shading needs be calculated. The array
!*	entries will contain the solar maximum solar elevation required to shade the cell at the 
!*	specified azimuth. This array will be called shading in the routine. The input, hsurf, is
!*	the standard elevation grid as is used throughout the program.
!*
!*	This program makes use of nrtype to make easy access double precision assignments
!*	
!*	This program also makes use of the following from params
!*		nlat 	either the number of rows or the number of latitudes in the grid space
!*		nlong	either the number of columns or the number of longitudes in the grid space
!*		latdist	either the grid spacing for a square grid, or the separation between grid entries
!*				along lines of latitude
!*		rad	converts degrees to radians
!*		nazbins	the number of azimuthal bins to compute shading for
!*	This program also uses the routine getdir to find the full paths of files to be opened
!*
!***************************************************************************************************


	REAL (DP),	DIMENSION(nlat,nlong),		    INTENT(IN   )	:: hsurf,hsurf_glac
	REAL (DP),	DIMENSION(nlat,nlong,36),	    INTENT(  OUT)	:: shading
	
	REAL (DP)								:: dlat=0._dp, &
											   dlong=0._dp, &
											   solaz=0._dp, &
											   elev1=0._dp, &
											   elev2=0._dp, &
		    								   elev3=0._dp, &
		    								   latindex=0._dp, &
		    								   longindex=0._dp, &
		    								   elevangle=0._dp, &
		    								   maxelevangle=0._dp, &
		    								   t, &
		    								   u, &
		    								   elevangle2=0._dp, &
		    								   maxelevangle2=0._dp

	INTEGER(I4B) :: h,i,j,k,l,m,n,o,maxsteps=1000,maxlat,minlat,maxlong,minlong

	shading(:,:,:)=0._dp
	DO h=1,36,1
	    solaz=(90._dp+REAL(10*(h-1)))
	    dlat=cell_size*SIN(solaz*rad)
	    dlong=-cell_size*COS(solaz*rad)
	    DO i=3,(nlat-2),1
			DO j=3,(nlong-2),1
				IF (hsurf_glac(i,j) .LT. 9000) THEN	!Not a blanked cell
					latindex=REAL(i)
					longindex=REAL(j)
					maxelevangle=0._dp
					maxelevangle2=0._dp
					elev1=hsurf(i,j)
					DO k=1,maxsteps,1
						latindex=latindex+(dlat/cell_size)
						longindex=longindex+(dlong/cell_size)			
						elevangle=0._dp
						IF ((CEILING(latindex) .GE. nlat) .OR. (FLOOR(latindex) .LE. 1) .OR. &
							(CEILING(longindex) .GE. nlong) .OR. (FLOOR(longindex) .LE. 1)) EXIT	
						maxlat=CEILING(latindex)
						minlat=FLOOR(latindex)
						IF (maxlat .EQ. minlat) maxlat=maxlat+1
						maxlong=CEILING(longindex)
						minlong=FLOOR(longindex)
						IF (maxlong .EQ. minlong) maxlong=maxlong+1
						CALL bilinterp(hsurf,longindex,latindex,minlong,maxlong,minlat,maxlat,elev2)
						!CALL bilinterp(hsurf,latindex,longindex,minlat,maxlat,minlong,maxlong,elev2)
			
						IF (elev1 .GE. elev2) THEN
							elevangle=0._dp
						ELSE
							elevangle=ATAN((elev2-elev1)/(SQRT((cell_size*(ABS(latindex)-REAL(i)))**2 &
							 +(cell_size*(ABS(longindex)-REAL(j)))**2)))
							IF (elevangle .GT. maxelevangle) maxelevangle=elevangle			
						END IF
					END DO
		    
					shading(i,j,h)=maxelevangle/rad
				ELSE
					shading(i,j,h)=0._dp
				END IF
			END DO
	    END DO
	END DO
	END SUBROUTINE shady
