SUBROUTINE slope_asp(hsurf,slope,aspect)
USE params
USE nrtype
IMPLICIT NONE

	!*********************************************************************
	!*
	!*  This is a program that calculates the slope and aspect of a given
	!*  elevation grid over spherical, earth coordinates using simple,
	!*  first order approximations to the spatial differential equations. 
	!*
	!* This program now has an option to do the calculation in either 
	!* spherical or orthogonal coordinate systems.
	!*
	!**********************************************************************

	REAL (DP), 		DIMENSION(1:nlat,1:nlong),			INTENT(IN   )	:: hsurf
	REAL (DP), 		DIMENSION(1:nlat,1:nlong),			INTENT(  OUT)	:: slope, &
										   aspect
	
	REAL (DP),		DIMENSION(2:nlat-1,2:nlong-1)						:: subgrid, &
																		   nsgrid, &
																		   ewgrid
	
	REAL (DP),		DIMENSION(1:nlat)			:: lats, &
									   londist, &
									   latdist
	
	REAL (DP)							:: north, &
									   south, &
									   east, &
									   west, &
									   deltalon=0.5

	INTEGER (I4B) 							:: i, &
									   j, &
									   k, &
									   l, &
									   m, &
									   n, &
									   o

	latdist(:)=cell_size
	
	IF (coordtype .EQ. "spher") THEN

	    DO i=1,nlat,1
		lats(i)=36.25_dp+(REAL(i)-1)*0.5_dp
   	    END DO
	    londist(:)=(deltalon*2._dp*pi*earthrad*COS(lats(:)*rad))/360._dp
	
	ELSE IF (coordtype .EQ. "ortho") THEN
	        londist(:)=latdist
	ELSE
	    PRINT *,"please enter grid format in parameters file!"
	    PRINT *,"Must be either spher or ortho."
	    STOP
	END IF
	
	!nsgrid=((hsurf(1:(nlat-3),2:(nlong-1))-hsurf(3:nlat,(2:nlong-1)))/(2._dp*latdist))
    !ewgrid=((hsurf(2:(nlat-1),1:(nlong-3))-hsurf(2:(nlat-1),3:nlong))/(2._dp*latdist))
	!slope(2:(nlat-1),2:(nlong-1))=ATAN(DSQRT(nsgrid**2 + ewgrid**2))
	!aspect(2:(nlat-1)),(2:nlong-1))=ATAN(ewgrid/nsgrid)
	DO i=2,nlat-1,1
	    DO j=2,nlong-1,1
	            north=hsurf(i+1,j)
		    south=hsurf(i-1,j)
		    east=hsurf(i,j+1)
		    west=hsurf(i,j-1)		    
				
		
		slope(i,j)=ATAN(DSQRT((((north-south)/(2*latdist(i)))**2)+(((east-west)/(2*londist(i)))**2)))
		

		IF ((ABS(north-south)) .LT. 0.001_dp) THEN
		    IF (east.GT.west) aspect(i,j)=pi/2.
		    IF (east.LT.west) aspect(i,j)=-pi/2.
		ELSE IF ((ABS(east-west)) .LT. 0.001) THEN
		    IF (north.EQ.south) aspect(i,j)=0.
		    IF (north.GT.south) aspect(i,j)=0.
		    IF (north.LT.south) aspect(i,j)=pi
		ELSE
		    aspect(i,j)=ATAN(((west-east)/(2*londist(i)))/((south-north)/(2.*latdist(i))))
		    IF ((south.GT.north).AND.(east.GT.west)) aspect(i,j)=aspect(i,j)+pi
		    IF ((south.GT.north).AND.(east.LT.west)) aspect(i,j)=aspect(i,j)-pi
		END IF
	    END DO
	END DO	

	
	
	slope(:,1)=0._dp
	slope(:,nlong)=0._dp
	slope(1,:)=0._dp
	slope(nlat,:)=0._dp
	aspect(:,1)=0._dp
	aspect(:,nlong)=0._dp
	aspect(1,:)=0._dp
	aspect(nlat,:)=0._dp

	END SUBROUTINE slope_asp
