PROGRAM terraincalc
USE nrtype
USE params
IMPLICIT NONE



!*******************************************************************************************
!*
!*  Enough is enough. This program will be the preprocessor for the energy balance model of
!*  Poellot, 2000. To unclutter that code, this program will calculate all of the slope and
!*  aspect data for a given DEM and output that to files that the main energy balance program
!*  can access whin the time comes. This preprocessor will also take care of generating the
!*  shading information and the solar geometry for later use. This will combine the code of
!*  FSA's thesis and that of Colin's.
!*
!*  This is the MAIN routine, which will call subroutines to calculate the essentials.
!*
!*  Watch this space for more details...
!*
!*
!*******************************************************************************************




	REAL(DP),	ALLOCATABLE,	DIMENSION(:,:)			:: hsurf, &
									   hsurf_glac, &
									   slope, &
								   aspect, &
								   meanhangle

	REAL(DP),	ALLOCATABLE,		DIMENSION(:,:,:)	:: shading

	!REAL(DP),	ALLOCATABLE,		DIMENSION(:,:)	:: solar_azimuth, &
	!							   solar_elevation

	REAL(DP)							:: zmin, &
									   zmax, &
									   trash, &
								   day, &
								   hour


	INTEGER(I4B)					:: h,i,j,k,l,ios,ios1,sdir,round

	CHARACTER(LEN=4)				:: DSAA
	CHARACTER(LEN=4)				:: filenumber
	CHARACTER(LEN=50)				:: filename
	CALL parameters()
!		DO h=1901,1957,1
			WRITE(filenumber,FMT='(i4)') h
			filename=TRIM(ADJUSTL(basegridname))  !//filenumber//'.grd'
			OPEN(UNIT=11,FORM="formatted",STATUS="old",FILE=inputstr//TRIM(ADJUSTL(filename)))
			filename=' '
			filename=TRIM(ADJUSTL(blankedgridname))
			!filename=filenumber//TRIM(ADJUSTL(blankedgridname))
			OPEN(UNIT=12,FORM="formatted",STATUS="old",FILE=inputstr//TRIM(ADJUSTL(filename)))
			DO i=1,2,1
				READ(UNIT=(10+i),FMT='(4a)',IOSTAT=ios) DSAA
				IF (ios .NE. 0) THEN
					PRINT *,"Error reading the base grid in terrain.exe"
					PRINT *,"Aborting...."
					STOP
				END IF
				READ(UNIT=(10+i),FMT=*) nlong,nlat
				READ(UNIT=(10+i),FMT=*) xmin,xmax
				READ(UNIT=(10+i),FMT=*) ymin,ymax
				READ(UNIT=(10+i),FMT=*) zmin,zmax
			END DO
			cell_size=(xmax-xmin)/(nlong-1)
			ALLOCATE(hsurf(nlat,nlong))
			ALLOCATE(hsurf_glac(nlat,nlong))
			ALLOCATE(slope(nlat,nlong))
			ALLOCATE(aspect(nlat,nlong))
			ALLOCATE(shading(nlat,nlong,36))
			ALLOCATE(meanhangle(nlat,nlong))
			!ALLOCATE(solar_azimuth(365,1:24))
			!ALLOCATE(solar_elevation(365,1:24))

			DO i=1,nlat,1
				READ(UNIT=11,FMT=*,IOSTAT=ios) (hsurf(i,j),j=1,nlong,1)
				READ(UNIT=12,FMT=*,IOSTAT=ios1) (hsurf_glac(i,j),j=1,nlong,1)
			END DO
			CLOSE(UNIT=11)
			CLOSE(UNIT=12)
			IF ((ios .NE. 0) .OR. (ios1 .NE. 0)) THEN
				PRINT *,"End of DEM file reached unexpectedly"
				PRINT *,"Proceeding with caution..."
			END IF

			!DO i=1,365,1
			!	DO j=0,23,1
			!		day=REAL(i)
			!		hour=REAL(j)
			!		CALL az_elev(day,hour,solar_azimuth(i,j+1),solar_elevation(i,j+1))
			!	END DO
			!END DO
			!solar_azimuth(:,:)=ABS(solar_azimuth(:,:)-180)

			WHERE (hsurf(:,:) .GT. 10000)
				hsurf(:,:)=0._dp
				hsurf_glac(:,:)=0._dp
			ENDWHERE

			CALL slope_asp(hsurf,slope,aspect)
			CALL shady(hsurf,hsurf_glac,shading)

			shading(:,:,:)=shading(:,:,:)*(pi/180._dp)
			CALL meanhangles(shading,hsurf_glac,slope,aspect,meanhangle)

			filename=TRIM(ADJUSTL(shadingfilename))
			!filename=filenumber//TRIM(ADJUSTL(shadingfilename))
			OPEN(UNIT=93,FORM="formatted",STATUS="unknown",FILE=inputstr//TRIM(ADJUSTL(filename)))
			!filename=filenumber//TRIM(ADJUSTL(solazfilename))
			!OPEN(UNIT=94,FORM="formatted",STATUS="unknown",FILE=inputstr//TRIM(ADJUSTL(filename)))
			!filename=filenumber//TRIM(ADJUSTL(solelfilename))
			!OPEN(UNIT=95,FORM="formatted",STATUS="unknown",FILE=inputstr//TRIM(ADJUSTL(filename)))
			filename=TRIM(ADJUSTL(slopegridname))
			!filename=filenumber//TRIM(ADJUSTL(slopegridname))
			CALL gridder(slope,filename)
			filename=TRIM(ADJUSTL(aspectgridname))
			!filename=filenumber//TRIM(ADJUSTL(aspectgridname))
			CALL gridder(aspect,filename)
			filename=TRIM(ADJUSTL(meanhanglegridname))
			!filename=filenumber//TRIM(ADJUSTL(meanhanglegridname))
			CALL gridder(meanhangle,filename)

			DO k=1,HC,1
				DO i=1,nlat,1
					WRITE(UNIT=93,FMT='(700(f5.2," "))') (shading(i,j,k),j=1,nlong,1)
				END DO
			END DO
			!DO i=1,365,1
			!	WRITE(UNIT=94,FMT='(24(F7.2," "))') (solar_azimuth(i,j),j=1,24,1)
			!	WRITE(UNIT=95,FMT='(24(F7.2," "))') (solar_elevation(i,j),j=1,24,1)
			!END DO
			CLOSE(UNIT=91)
			CLOSE(UNIT=92)
			CLOSE(UNIT=93)
			!CLOSE(UNIT=94)
			!CLOSE(UNIT=95)
			CLOSE(UNIT=96)
			DEALLOCATE(hsurf)
			DEALLOCATE(hsurf_glac)
			DEALLOCATE(slope)
			DEALLOCATE(aspect)
			DEALLOCATE(shading)
			DEALLOCATE(meanhangle)
			!DEALLOCATE(solar_azimuth)
			!DEALLOCATE(solar_elevation)
!		ENDDO

END PROGRAM terraincalc
