SUBROUTINE turbcalc(AVP,svp_air,atemp,wind,P,snowd,stemp,Hs,Hl,trans_coeff_snow,trans_coeff_ice,eh,vpdiff)
USE nrtype
USE params
IMPLICIT NONE
	

!*********************************
!* Turbulent Heat Flux Equations *
!*********************************
!c
!c
!c calculation of Turbulent heat fluxes
!c  equations from Paterson, 1994 (pp.64-65, eqs.19,20);
!c  and Braithwaite, et.al., 1998 (calculations of Lv and cp)



	REAL(DP),						INTENT(INOUT)	:: AVP
	REAL(DP),						INTENT(INOUT)	:: svp_air
	REAL(DP),						INTENT(IN   )	:: atemp
	REAL(DP),						INTENT(IN   )	:: wind
	REAL(DP),						INTENT(IN   )	:: P
	REAL(DP),						INTENT(IN   )	:: snowd
	REAL(DP),						INTENT(IN   )	:: stemp
	REAL(DP),	DIMENSION(80,200),	INTENT(IN   )	:: trans_coeff_snow,trans_coeff_ice
	REAL(DP),						INTENT(  OUT)	:: Hs
	REAL(DP),						INTENT(  OUT)	:: Hl
	REAL(DP),						INTENT(  OUT)   :: eh
	REAL(DP),						INTENT(  OUT)	:: vpdiff

	REAL(DP)					:: SVP, &
							   ustar, &
					   		   zo, &
					   		   adj_density, &
					   		   lheat, &
							   hs_old, &
							   ustar_old, &
					   		   molength_old, &
					   		   eh_old, &
					   		   meantemp, &
							   rho_cp_t_over_g_k, &
							   rho_cp_wind_deltat, &
							   psi, &
							   molength
							   	
	INTEGER(I4B)				:: round, &
								   tindex, &
								   windex, &
								   i


	adj_density=P*sea_density/sea_press
	IF (AVP .GT. svp_air) THEN
		AVP=svp_air
		cloudy_q=1
	ELSE
		cloudy_q=0
	ENDIF			!that is being modelled with the current turb fluxes	
	IF (atemp.gt.273.15) THEN !Oerlemans, 1992		!these things were great enough that they should be tracked in terms of their mass balance influence.
		SVP=610.78   !This is the Fixed saturation VP for a melting ice surface.  !*exp(19.85*(1-(273.15/Atemp)))
	ELSE
		SVP=610.78*exp(17.2693882*(stemp-273.16)/(stemp-35.86))
		!PRINT *,SVP,AVP,SVP-AVP
		!PAUSE
	ENDIF	  
	meantemp=(atemp+stemp)/2
	SELECT CASE(model)
	CASE(1)
		Hs=Khs*(Atemp-Stemp) !Oerlemans eq.s
		Hl=Khs*0.622*Lf*(AVP-SVP)/(P*Cp) 
	CASE(2)
		!Paterson eq.s
		IF (snowd.gt.0.) THEN
		Hs=1.29e-2*Khs*P*wind*(atemp-stemp) !This model needs to be calculating surface temepratures
		IF(AVP.lt.SVP)THEN !evaporation		!As currently formulated, it's predicting
							!downward latent heat flux in winter, which is unlikely 
			Hl=22.56*Khs*wind*(AVP-SVP)		!These equations should be avp and surface vp, not
		ELSE		   !condensation	!saturation vapor pressures...
			Hl=19.91*Khs*wind*(AVP-SVP)
		ENDIF
		ELSE
		Hs=1.29e-2*Khi*P*wind*(atemp-stemp)
		IF(AVP.lt.SVP)then !evaporation
			Hl=22.2*Khi*wind*(AVP-SVP)
		ELSE
			Hl=19.91*Khi*wind*(AVP-SVP)
		ENDIF
		ENDIF
	CASE(3)
		IF (snowd.gt.0.) THEN
		Hs=4.42e-6*P*(atemp-stemp)*wind
		IF (AVP.lt.SVP) THEN !evaporation
			Hl=7.77e-3*(avp-svp)*wind 
		ELSE			!condensation
			Hl=6.86e-3*(avp-svp)*wind 
		ENDIF
		ELSE
		Hs=6.34e-6*P*(atemp-stemp)*wind
		IF (AVP.lt.SVP) THEN
			Hl=11.14e-3*(avp-svp)*wind 
		ELSE
			Hl=9.83e-3*(avp-svp)*wind 
		ENDIF
		ENDIF
	CASE(4)
		!Model 4 will be the monin-obukhof similarity theory SHF and LHF calculations
		!This is going to require iteration to solve and will need to be able to treat 
		!surface roughness and eventually something about surface temperature. However, this will
		!usually be equal to air temperature or the freezing point. Shouldn't change except maybe
		!at night and diurnal fluctuation isn't treated here.
		IF (avp .GT. svp) THEN

			!condensing, need to use the latent heat of vaporisation
			lheat=lv
		ELSE
			!evaporating, need to use the latent heat of sublimation
			lheat=ls
		ENDIF

		IF (snowd.gt.0.) THEN
			zo=zo_snow
		ELSE
			zo=zo_ice
		ENDIF
		! First, calculate initial values for A, SHF, obukhof length, and ustar
		hs_old=0._dp
		ustar_old=0._dp
		molength_old=0._dp
		rho_cp_t_over_g_k=adj_density*cp_air*meantemp/(gravity*vonkarman)
		rho_cp_wind_deltat=cp_air*adj_density*wind*(atemp-stemp)
		eh_old=(vonkarman/(LOG(inst_height/zo)))**2
		ustar_old=vonkarman*wind/LOG(inst_height/zo)
		hs_old=rho_cp_wind_deltat*eh_old
		IF (hs_old .EQ. 0._dp) hs_old=1._dp	!Assign a non-zero, arbitrary hs to keep things alive when conditions are neutral
		molength_old=rho_cp_t_over_g_k*(ustar_old**3)/(hs_old)
		DO i=1,maxiterations,1	!begin the iterative loop for Calculating SHF and LHFs
			eh=(vonkarman/(LOG(inst_height/zo)+psi(inst_height,molength_old) &
			*(inst_height/molength_old)))**2
			hs=rho_cp_wind_deltat*eh
			IF (hs .EQ. 0._dp) hs=1._dp
			IF (ABS(hs) .GT. 1000) hs=1._dp
			ustar=wind*vonkarman/(LOG(inst_height/zo)+psi(inst_height,molength_old) &
					*(inst_height/molength_old))
			molength=rho_cp_t_over_g_k*(ustar**3)/(hs)
			IF ((ABS((molength-molength_old)/molength) .LT. turbflux_tol) .AND. (i .GT. 2)) THEN	!End the iteration as it has converged	
				eh=(vonkarman/(LOG(inst_height/zo)+psi(inst_height,molength_old) &
				*(inst_height/molength)))**2
				hs=rho_cp_wind_deltat*eh
				IF (hs .EQ. 0._dp) hs=1._dp
				hl=lheat*adj_density*gasconst_ratio*eh*wind*(AVP-SVP)/P
				eh_old=eh
				ustar_old=ustar
				molength_old=molength
				hs_old=hs	
				EXIT
			ELSE
				IF (i .EQ. maxiterations) THEN
					eh=(vonkarman/(LOG(inst_height/zo)))**2
					IF (wind .LT. 4._dp) eh=0._dp
					hs=adj_density*cp_air*wind*eh*(atemp-stemp)
					hl=lheat*adj_density*gasconst_ratio*eh*wind*(AVP-SVP)/P
					eh_old=eh
					ustar_old=ustar
					molength_old=molength
					hs_old=hs
					EXIT
				END IF
				eh_old=eh
				ustar_old=ustar
				molength_old=molength
				hs_old=hs
			ENDIF
		ENDDO
	CASE(5)
		IF (avp .GT. svp) THEN
			!condensing, need to use the latent heat of vaporisation
			lheat=lv
		ELSE
			!evaporating, need to use the latent heat of sublimation
			lheat=ls
		ENDIF
		windex=round(wind*4._dp)
		tindex=round(((atemp-273.16)+20)*4)
		IF (wind .GE. 20) windex=80
		IF (atemp-273.16 .GE. 30) tindex=200
		IF ((windex .LE. 0) .OR. (tindex .LE. 0)) THEN
			hs=0._dp
			eh=0._dp
			windex=1
			DO i=1,20,1
				IF (snowd.gt.0.) THEN
					eh=trans_coeff_snow(windex,1)
				ELSE
					eh=trans_coeff_ice(windex,1)
				ENDIF
				IF (eh .GT. 0._dp) THEN
					EXIT
				ENDIF
				windex=windex+1
			ENDDO
			hl=lheat*adj_density*gasconst_ratio*eh*wind*(AVP-SVP)/P
			vpdiff=hl
			RETURN
		ENDIF
		eh=0._dp
		DO i=1,20,1
			IF (snowd.gt.0.) THEN
				eh=trans_coeff_snow(windex,tindex)
			ELSE
				eh=trans_coeff_ice(windex,tindex)
			ENDIF
			IF (eh .GT. 0._dp) THEN
				EXIT
			ENDIF
			windex=windex+1
		ENDDO
		rho_cp_wind_deltat=cp_air*adj_density*wind*(atemp-stemp)
		hs=rho_cp_wind_deltat*eh
		hl=lheat*adj_density*gasconst_ratio*eh*wind*(AVP-SVP)/P
		vpdiff=hl
			
	END SELECT
	
	RETURN
END SUBROUTINE turbcalc
